/*******************************************************************
|*
|*  Version:    1.3
|*
|*  Description:
|*
|*      TASKING M16C ROM Monitor: commands and arror codes
|*
|*  Copyright 2002, Altium BV
|*
 *******************************************************************/

#ifndef _M16C_H
#define _M16C_H

#include <string.h>

#define REG_R0   0x00
#define REG_R1   0x01
#define REG_R2   0x02
#define REG_R3   0x03
#define REG_A0   0x04
#define REG_A1   0x05
#define REG_FB   0x06
#define REG_R0B  0x07
#define REG_R1B  0x08
#define REG_R2B  0x09
#define REG_R3B  0x0a
#define REG_A0B  0x0b
#define REG_A1B  0x0c
#define REG_FBB  0x0d
#define REG_USP  0x0e
#define REG_ISP  0x0f
#define REG_FLG  0x10
#define REG_PC   0x11
#define REG_SB   0x12
#define REG_INTB 0x13

#define REG_MAX  0x14

/* 
 * defines, typedefs, enums
 */

enum {GO, STEP }; /* go or stepping */


/*
 * global variables
 */
extern volatile __near unsigned char step_or_go;
extern const unsigned char   hex[];
extern __near unsigned long   dump_address;
extern __near unsigned char   application_running;
extern __near unsigned int   * __near isp_user;
extern __near unsigned int   * __near isp_monitor;
extern __near unsigned char   user_mode;

extern volatile __near unsigned int step_count;
extern volatile __near unsigned char step_or_go;
extern volatile __near unsigned long pc_pc;
extern volatile __near unsigned long pc_rmad0;
extern volatile __near unsigned long pc_rmad1;
#ifdef HAS_4BREAKPOINTS
extern volatile __near unsigned long pc_rmad2;
extern volatile __near unsigned long pc_rmad3;
#endif

/*
 * global functions, inline functions
 */
void m16c_fill(unsigned long address, unsigned int length, unsigned char data_type, unsigned int data);
void m16c_dump(unsigned int length, unsigned char data_type);
unsigned char m16c_step(unsigned int * number);
void m16c_erase_flash(void);
void m16c_write_flash(unsigned long address, __near unsigned char *buf);

extern unsigned long m16c_dump_reg(unsigned char reg_number);
extern void          m16c_set_reg(unsigned char reg_number, unsigned long reg_value);
extern unsigned long m16c_get_PC( void );
extern void          m16c_set_PC( unsigned long pc );
extern unsigned int  m16c_get_FLG( void );
extern void          m16c_set_FLG( unsigned int flg );

inline void switch_to_user_mode( void )
{
    user_mode = 1;
    isp_monitor = (__near unsigned int *)__stc_isp();
    __ldc_isp( (unsigned int)isp_user );
}

inline void switch_to_monitor_mode( void )
{
    user_mode = 0;
    isp_user = (__near unsigned int *)__stc_isp();
    __ldc_isp( (unsigned int)isp_monitor );
}

inline void asmgetR0( void )
{
    __asm("        mov.w   R0,_val4");
}
inline void asmgetR1( void )
{
    __asm("        mov.w   R1,_val4");
}
inline void asmgetR2( void )
{
    __asm("        mov.w   R2,_val4");
}
inline void asmgetR3( void )
{
    __asm("        mov.w   R3,_val4");
}
inline void asmgetA0( void )
{
    __asm("        mov.w   A0,_val4");
}
inline void asmgetA1( void )
{
    __asm("        mov.w   A1,_val4");
}
inline void asmgetFB( void )
{
        val4 = __stc_fb();
}

#endif /* _M16C_H */
