/********************************************************************
|*
|*  Version:    1.4
|*
|*  Description:
|*
|*      Prototypes, definitions and typedefs for the UART driver
|*
|*  Copyright 2002, Altium BV
|*
 ********************************************************************/

#ifndef _UART_H
#define _UART_H

/*
 * Used interrupt numbers
 */

// m30245 has a 4th UART, and different receive IRQ numbers. Transmit is the same.

#ifdef HAS_UART3
# define RXD0IRQ         10              // IRQ of UART0 receiver
# define RXD1IRQ          8              // IRQ of UART1 receiver
# define RXD2IRQ          2              // IRQ of UART2 receiver
# define RXD3IRQ         21              // IRQ of UART3 receiver
# define TXD3IRQ         13              // IRQ of UART3 transmitter
# define SERIAL_INT0_STR "0A"            // IRQ of UART0 transmitter (string in hex format!)
# define SERIAL_INT1_STR "08"            // IRQ of UART1 transmitter (string in hex format!) 
# define SERIAL_INT2_STR "02"            // IRQ of UART2 transmitter (string in hex format!)
# define SERIAL_INT3_STR "0D"            // IRQ of UART3 transmitter (string in hex format!)
#else
# define RXD0IRQ         18              // IRQ of UART0 receiver
# define RXD1IRQ         20              // IRQ of UART1 receiver
# define RXD2IRQ         16              // IRQ of UART2 receiver

# define SERIAL_INT0_STR "11"            // IRQ of UART0 transmitter (string in hex format!)
# define SERIAL_INT1_STR "13"            // IRQ of UART1 transmitter (string in hex format!) 
# define SERIAL_INT2_STR "0F"            // IRQ of UART2 transmitter (string in hex format!)

#endif

#define TXD0IRQ         17              // IRQ of UART0 transmitter
#define TXD1IRQ         19              // IRQ of UART1 transmitter
#define TXD2IRQ         15              // IRQ of UART2 transmitter



/* 
 * defines, typedefs, enums
 */
#if UART == 0
#  define UxTB          U0TB
#  define UxTBL         U0TBL
#  define UxTBH         U0TBH
#  define UxRB          U0RB
#  define UxRBL         U0RBL
#  define UxRBH         U0RBH
#  define UxBRG         U0BRG
#  define UxMR          U0MR
#  define UxC0          U0C0
#  define UxC1          U0C1
#  define UxC1_TI       U0C1_TI
#  define UxC1_RI       U0C1_RI
#  define UxC1_RE       U0C1_RE
#  define SxRIC         S0RIC
#  define TxRIC         T0RIC
#  define RXDxIRQ       RXD0IRQ
#  define TXDxIRQ       TXD0IRQ
#  define SxTIC_IR      S0TIC_IR
#  define SxRIC_IR      S0RIC_IR
#  define SERIAL_INT_STR        SERIAL_INT0_STR
#elif UART == 1
#  define UxTB          U1TB
#  define UxTBL         U1TBL
#  define UxTBH         U1TBH
#  define UxRB          U1RB
#  define UxRBL         U1RBL
#  define UxRBH         U1RBH
#  define UxBRG         U1BRG
#  define UxMR          U1MR
#  define UxC0          U1C0
#  define UxC1          U1C1
#  define UxC1_TI       U1C1_TI
#  define UxC1_RI       U1C1_RI
#  define UxC1_RE       U1C1_RE
#  define SxRIC         S1RIC
#  define TxRIC         T1RIC
#  define RXDxIRQ       RXD1IRQ
#  define TXDxIRQ       TXD1IRQ
#  define SxTIC_IR      S1TIC_IR
#  define SxRIC_IR      S1RIC_IR
#  define SERIAL_INT_STR        SERIAL_INT1_STR
#elif UART == 2
#  define UxTB          U2TB
#  define UxTBL         U2TBL
#  define UxTBH         U2TBH
#  define UxRB          U2RB
#  define UxRBL         U2RBL
#  define UxRBH         U2RBH
#  define UxBRG         U2BRG
#  define UxMR          U2MR
#  define UxC0          U2C0
#  define UxC1          U2C1
#  define UxC1_TI       U2C1_TI
#  define UxC1_RI       U2C1_RI
#  define UxC1_RE       U2C1_RE
#  define SxRIC         S2RIC
#  define TxRIC         T2RIC
#  define RXDxIRQ       RXD2IRQ
#  define TXDxIRQ       TXD2IRQ
#  define SxTIC_IR      S2TIC_IR
#  define SxRIC_IR      S2RIC_IR
#  define SERIAL_INT_STR        SERIAL_INT2_STR
#ifdef HAS_UART3
# elif UART == 3
#  define UxTB          U3TB
#  define UxTBL         U3TBL
#  define UxTBH         U3TBH
#  define UxRB          U3RB
#  define UxRBL         U3RBL
#  define UxRBH         U3RBH
#  define UxBRG         U3BRG
#  define UxMR          U3MR
#  define UxC0          U3C0
#  define UxC1          U3C1
#  define UxC1_TI       U3C1_TI
#  define UxC1_RI       U3C1_RI
#  define UxC1_RE       U3C1_RE
#  define SxRIC         S3RIC
#  define TxRIC         T3RIC
#  define RXDxIRQ       RXD3IRQ
#  define TXDxIRQ       TXD3IRQ
#  define SxTIC_IR      S3TIC_IR
#  define SxRIC_IR      S3RIC_IR
#  define SERIAL_INT_STR        SERIAL_INT3_STR
#endif
#else
#  error Unsupported UART defined
#endif


#define UART_ERROR      0xF900          // ANDing uart_getchar() results in error status
#define UART_MASK       0x00FF          // ANDing uart_getchar() results in character value

#define UART_EMPTY      0x0100          // UART has no character available
#define UART_ARB        0x0800          // UART arbitration lost (not possible)
#define UART_OER        0x1000          // UART overrun error
#define UART_FER        0x2000          // UART framing error
#define UART_PER        0x4000          // UART parity error
#define UART_SUM        0x7000          // UART error (either overrun, framing or parity)

#define XON                0x11 /* ^Q */
#define XOFF               0x13 /* ^S */

#define INPUT_BUFFER_SIZE  20

#define LOW_WATER_MARK     10 /* low water mark for flow control */
#define HIGH_WATER_MARK    15 /* high water mark for flow control */

/*
 * global variables
 */
extern void           uart_init( void );                      // Initialize serial port
extern void           uart_putchar( unsigned char byte );     // Write a character
extern unsigned char  uart_getchar( void );                   // Read a character
extern void           uart_puts( const char * str );          // Write an ASCIIZ string on the serial port
 
extern volatile __near unsigned char  flag_xoff_sent;

extern volatile __near unsigned char  se_buf[];
extern volatile __near char           se_buf_in;
extern volatile __near char           se_buf_out;

#endif /* _UART_H */
